# Generated by roxygen2: do not edit by hand

export(available_dates)
export(available_df)
export(available_r)
export(ga_cachix)
export(make_launcher)
export(nix_build)
export(renv2nix)
export(rix)
export(rix_init)
export(setup_cachix)
export(tar_nix_ga)
export(with_nix)
importFrom(codetools,checkUsage)
importFrom(codetools,findGlobals)
importFrom(curl,curl_fetch_disk)
importFrom(curl,curl_fetch_memory)
importFrom(curl,handle_reset)
importFrom(curl,handle_setheaders)
importFrom(curl,has_internet)
importFrom(curl,new_handle)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,read_json)
importFrom(stats,na.omit)
importFrom(tools,pskill)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,untar)
