## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----include=FALSE------------------------------------------------------------
library(rix)

## ----eval = F-----------------------------------------------------------------
# path_default_nix <- tempdir()
# 
# rix(
#   r_ver = "latest-upstream",
#   r_pkgs = c("quarto"),
#   system_pkgs = c("quarto", "git"),
#   git_pkgs = NULL,
#   ide = "none",
#   project_path = path_default_nix,
#   overwrite = TRUE
# )

## ----parsermd-chunk-2, echo = F-----------------------------------------------
#> # This file was generated by the {rix} R package v0.7.1 on 2024-07-01
#> # with following call:
#> # >rix(r_ver = "12a9c0004bc987afb1ff511ebb97b67497a68e22",
#> #  > r_pkgs = c("quarto"),
#> #  > system_pkgs = c("quarto",
#> #  > "git"),
#> #  > git_pkgs = NULL,
#> #  > ide = "none",
#> #  > project_path = path_default_nix,
#> #  > overwrite = TRUE)
#> # It uses nixpkgs' revision 12a9c0004bc987afb1ff511ebb97b67497a68e22 for reproducibility purposes
#> # which will install R version latest.
#> # Report any issues to https://github.com/ropensci/rix
#> let
#>  pkgs = import (fetchTarball "https://github.com/NixOS/nixpkgs/archive/12a9c0004bc987afb1ff511ebb97b67497a68e22.tar.gz") {};
#>
#>   rpkgs = builtins.attrValues {
#>     inherit (pkgs.rPackages)
#>       quarto;
#>   };
#>
#>   system_packages = builtins.attrValues {
#>     inherit (pkgs)
#>       quarto
#>       git
#>       R
#>       glibcLocales
#>       nix;
#>   };
#>
#> in
#>
#> pkgs.mkShell {
#>   LOCALE_ARCHIVE = if pkgs.system == "x86_64-linux" then  "${pkgs.glibcLocales}/lib/locale/locale-archive" else "";
#>   LANG = "en_US.UTF-8";
#>    LC_ALL = "en_US.UTF-8";
#>    LC_TIME = "en_US.UTF-8";
#>    LC_MONETARY = "en_US.UTF-8";
#>    LC_PAPER = "en_US.UTF-8";
#>    LC_MEASUREMENT = "en_US.UTF-8";
#>
#>   buildInputs = [  rpkgs  system_packages   ];
#>
#> }

## ----eval = F-----------------------------------------------------------------
# path_default_nix <- tempdir()
# 
# rix(
#   r_ver = "latest-upstream",
#   r_pkgs = c("quarto"),
#   system_pkgs = "quarto",
#   tex_pkgs = c("amsmath"),
#   ide = "none",
#   project_path = path_default_nix,
#   overwrite = TRUE,
#   print = TRUE
# )
# #> # This file was generated by the {rix} R package v0.7.1 on 2024-07-01
# #> # with following call:
# #> # >rix(r_ver = "12a9c0004bc987afb1ff511ebb97b67497a68e22",
# #> #  > r_pkgs = c("quarto"),
# #> #  > system_pkgs = "quarto",
# #> #  > tex_pkgs = c("amsmath"),
# #> #  > ide = "none",
# #> #  > project_path = path_default_nix,
# #> #  > overwrite = TRUE,
# #> #  > print = TRUE)
# #> # It uses nixpkgs' revision 12a9c0004bc987afb1ff511ebb97b67497a68e22 for reproducibility purposes
# #> # which will install R version latest.
# #> # Report any issues to https://github.com/ropensci/rix
# #> let
# #>  pkgs = import (fetchTarball "https://github.com/NixOS/nixpkgs/archive/12a9c0004bc987afb1ff511ebb97b67497a68e22.tar.gz") {};
# #>
# #>   rpkgs = builtins.attrValues {
# #>     inherit (pkgs.rPackages)
# #>       quarto;
# #>   };
# #>
# #>   tex = (pkgs.texlive.combine {
# #>     inherit (pkgs.texlive)
# #>       scheme-small
# #>       amsmath;
# #>   });
# #>
# #>   system_packages = builtins.attrValues {
# #>     inherit (pkgs)
# #>       quarto
# #>       R
# #>       glibcLocales
# #>       nix;
# #>   };
# #>
# #> in
# #>
# #> pkgs.mkShell {
# #>   LOCALE_ARCHIVE = if pkgs.system == "x86_64-linux" then  "${pkgs.glibcLocales}/lib/locale/locale-archive" else "";
# #>   LANG = "en_US.UTF-8";
# #>    LC_ALL = "en_US.UTF-8";
# #>    LC_TIME = "en_US.UTF-8";
# #>    LC_MONETARY = "en_US.UTF-8";
# #>    LC_PAPER = "en_US.UTF-8";
# #>    LC_MEASUREMENT = "en_US.UTF-8";
# #>
# #>   buildInputs = [  rpkgs tex system_packages   ];
# #>
# #> }

